#ifndef _BSShell_interface_
#define _BSShell_interface_

#include <stdio.h>
#include "Parser.hh"
#include "buildin/Builtin.hh"
#include "memory/MemoryManager.hh"

/**
 * BSSShell ist ein Kommandozeileninterpreter. 
 * Nach seiner Initialisierung startet er eine Schleife, die erst durch den
 * Aufruf von <code>exit</code> oder einen Fehler beendet wird. In dieser Schleife
 * wird eine Kommandozeile eingelesen und analysiert, um danach eine dem 
 * eingelesenen Kommando enstsprechende Methode auszufhren.
 *
 * @author Arbeitsgruppe EOS
 */
class BSShell : public Parser {
private:
    /**
     * bestimmt den Prompt der Kommandozeile des Parsers
     */
    const char *_prompt;
    
    /**
     * verwaltet die internen Kommandos des Parsers
     */
    Builtin _internal_cmd;
    
    /**
     * verwaltet die internen Kommandos des Parsers
     */
    MemoryManager _memmanager;
     
    /**
     * Liest eine Kommandozeile ein. 
     *
     * @param buffer Pointer auf den Puffer, in den die Kommandozeile eingelesen wird
     * @param size Gre des bergebenen Puffers
     * @return Liefert die Anzahl der eingelesenen Zeichen oder einen Fehlercode.
     */
    int readline(char *buffer, const int size);
    
    /**
     * Fhrt die eingelesenen Kommandos im Interpreter aus.
     * Die Methode <code>execute()</code> fhrt ausschlielich interne Kommandos
     * aus, das Ausfhren von Kommandos, die nicht im Interpreter implementiert
     * sind, ist nicht vorgesehen.
     *
     * @return Liefert true, wenn das Kommando <code>exit</code> aufgerufen
     *         wurde und der Interpreter beendet werden soll.
     */
    bool execute();
public:
    /**
     * Erzeugt eine Instanze des Interpreters. Die erzeugte Instanze benutzt keinen 
     * Speichermanager.
     *
     * @param prompt Prompt der Kommandozeile des Parsers
     */
    BSShell(const char *prompt = "bssh> ");
    
    /**
     * Erzeugt eine Instanze des Interpreters. 
     *
     * @param memadr Startadresse des zu verwaltenden Speicherbereichs
     * @param memsize Gre des zu verwaltenden Speicherbereichs
     * @param prompt Prompt der Kommandozeile des Parsers
     */
    BSShell(char* astrategy, caddr_t memadr, size_t memsize, const char *prompt = "bssh> ");
    ~BSShell();
    
    /**
     * Startet die Ausfhrungsschleife des Interpreters.
     * Die Bearbeitung erfolgt dabei nach dem folgenden Schema: Einlesen und 
     * Analysieren einer Kommandozeile und danach Ausfhren der eingelesenen
     * Kommandos. Die Schleife wird solange abgearbeitet, bis der Interpreter
     * durch den Aufruf von <code>exit</code> oder einen Fehler beendet wird.
     *
     * @return Liefert true, wenn der Interpreter mit dem Aufruf von 
     *         <code>exit</code> benendet wurde.
     */
    bool loop();
    
    /**
     * The malloc() function allocates size bytes of memory. The allocated
     * space is suitably aligned (after possible pointer coercion) for storage
     * of any type of object. If the space is at least pagesize bytes in length
     * (see getpagesize()), the returned memory will be page boundary aligned
     * as well. If malloc() fails, a NULL pointer is returned.
     *
     * <strong>Hinweis:</strong> Im Normalfall initialisiert malloc() den 
     *                           allokierten Speicherbereich NICHT mit 0 Bytes.
     *
     * @param size Gre des zu allokierenden Speicherbereichs
     * @return Liefert die Adresse des allokierten Speicherbereichs oder NULL 
     *         wenn malloc() fehlschlgt.
     */
    void* malloc(size_t size);
    
    /**
     * The calloc() function allocates space for number objects, each size bytes
     * in length. The result is identical to calling malloc() with an argument
     * of <em>"number * size"</em>, with the exception that the allocated memory is
     * explicitly initialized to zero bytes.
     *
     * @param number Anzahl der zu allokierenden Speicherbereiche
     * @param size Gre eines zu allokierenden Speicherbereichs
     * @return Liefert die Adresse des allokierten Speicherbereichs oder NULL 
     *         wenn calloc() fehlschlgt.
     */
    void* calloc(size_t number, size_t size);
    
    /**
     * The realloc() function changes the size of previously allocated memory 
     * referenced by ptr to size bytes. The contents of the memory are
     * unchanged up to the lesser of the new and old sizes.  If the new size is
     * larger, the value of the newly allocated portion of the memory is 
     * undefined. If the requested memory cannot be allocated, NULL is returned
     * and the memory referenced by ptr is valid and unchanged. If memory can
     * be allocated, the memory referenced by ptr is freed and a pointer to the
     * newly allocated memory is returned. <strong>Note</strong> that this may
     * be different from the value passed as ptr. If ptr is NULL, the realloc()
     * function behaves identically to malloc() for the specified size.
     *
     * @param ptr Adresse des Speicherbereichs
     * @param size neue Gre des Speicherbereichs in Bytes
     * @return Liefert die Adresse des reallokierten Speicherbereichs oder NULL 
     *         wenn realloc() fehlschlgt.
     */
    void* realloc(void *ptr, size_t size);
    
    /**
     * Gibt eine allokierten Speicherbereich wieder frei. 
     * Die Adresse des freizugebenden Speicherbereichs wird als Parameter 
     * bergeben. Ist die Adresse gleich NULL wird keine Aktion ausgefhrt.
     *
     * @param ptr Adresse des Speicherbereichs
     */
    void free(void *ptr);    
    
    /**
     * Gibt die aktuelle Freispeicherlist auf dem 'ErrorStream' aus. 
     * Diese Methode dient nur zum Debuggen und Testen der Speicherstrategieen.
     */
    void showBlocks();

    /**
     * Gibt die Gre des zur Verfgung stehenden Speichers zurck.
     *
     * @return Liefert die Gre des verwalteten Speichers.
     */
    size_t size();    
    
    /**
     * Gibt die Gre des belegten Speichers zurck.
     *
     * @return Liefert die Gre des belegten Speichers
     */
    size_t used();    
};

#endif // _BSShell_interface_
